// ##########################################################################
// ####                                                                  ####
// ####                      RTP Audio Server Project                    ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### Scan URL                                                         ####
// ####                                                                  ####
// #### Version 1.00  --  February 16, 2001                              ####
// ####                                                                  ####
// #### Copyright (C) 1999  Thomas Dreibholz                             ####
// ####               2000  Universitt Bonn, Abt. IV                    ####
// ####               2001  EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################


import java.lang.*;


/**
  * This class extracts an URL into protocol, host and path.
  *
  * @short   Scan URL
  * @author  Thomas Dreibholz (Dreibholz@bigfoot.com)
  * @version 1.0
  */
public class ScanURL
{
   // ====== Variables ======================================================
   private String protocol = "";
   private String host     = "";
   private String path     = "";


   // ###### Constructor ####################################################
   /**
     * Constructor.
     */
   public ScanURL() {
   }
   

   // ###### Scan URL #######################################################
   /**
     * Scan an URL string and extract URL into protocol, host and path.
     *
     * @param location String with URL (e.g. rtpa://gaffel:7500/Test.list).
     * @param protocol Default protocol.
     * @return true, if URL scan was successful; false otherwise.
     */
   public boolean scanURL(String location, String defaultProtocol) {
      boolean ok = true;
      protocol   = defaultProtocol;

      // ====== Get protocol ================================================
      int p1 = location.indexOf("://");
      if(p1 < 0) {
         if(protocol.equals("")) {
            ok = false;
         }
         p1 = 0;
      }
      else {
         protocol = location.substring(0,p1);
         p1 += 3;
      }

      // ====== Get host ====================================================
      if(ok == true) {
         String url = location.substring(p1);
         int p2 = url.indexOf('/');
         if(p2 < 0) {
            ok = false;
         }
         else {
            host = url.substring(0,p2);

            // ====== Get path ==============================================
            path = url.substring(p2 + 1);
         }
      }

      protocol = protocol.toLowerCase();
      host     = host.toLowerCase();
      return(ok);
   }
   
   
   // ###### Get protocol ###################################################
   public String getProtocol() {
      return(protocol);
   }

   
   // ###### Get host #######################################################
   public String getHost() {
      return(host);
   }

   // ###### Get path #######################################################
   public String getPath() {
      return(path);
   }
}
