#include "flow.h"

namespace Coral {

Flow::Flow(TransportInfo2 _ti, QoSManagerInterface* interface, QoSManagerBase* _qm, card32* delayOfClass_, card32* lossRateOfClass_)
  : Synchronizable()
{
  delayOfClass = delayOfClass_;
  lossRateOfClass = lossRateOfClass_;

  ti = _ti;
  Interface = interface;
  qm = _qm;
  level = 255;
  current_bw = 0;
  wanted_bw = _ti.WantedBytesPerSecond.getValue();
  maxDelay = _ti.WantedMaxTransferDelay;
}

/*
Flow::~Flow()
{
  int i;
  Level* levelP = &(ti.QualityLevel[level]);
  card8 prioClass;
  card64 bw;

  for(i=0; i < (int) levelP->layer.size(); i++) {
    prioClass = levelP->layer[i].priorityClass;
    bw        = levelP->layer[i].getBandwidth(ti.start_fps * levelP->fps_scale);
    qm->freeInPrio[ prioClass ] += bw;
  }
}
*/

bool Flow::scaleUp(card32 sessionID)
{
  cout << "scale up flow " << flowID << " of session " << sessionID << endl;
  // cout << endl <<  "Scale up flow " << flowID << endl;
  int i,j;                             // counter

  card8 oldLevel = level;              // to remember the current level
  card8 newLevel;                      // new quality level
  
  // cout << "old level: " << (int) oldLevel << endl;

  // set new level
  if(level == 255) {
    newLevel = 0;
  } else {
    newLevel = ti.QualityLevel[level].LevelUp;
  }

  if(newLevel == 255) {
    return false;                                   // there is no next level
  }
 
  Level* newLevelP = &(ti.QualityLevel[newLevel]);  // Pointer to the new level.
  Level* oldLevelP = NULL;

  if(oldLevel != 255) {                            
    oldLevelP = &(ti.QualityLevel[oldLevel]);       // Pointer to the old level.
  }


  card8 numLayers = newLevelP->layer.size();        // number of layers in new level

  vector<bool> layerFit(numLayers);                 // to remember, wich layers fit
  vector<card64> layer_bw(numLayers);               // bandwidth of each new layer

  card64 level_bw = 0;                              // bandwidth, needed by the new level

  for(i=0; i < numLayers; i++) {               
    layerFit[i] = false;
    layer_bw[i] = newLevelP->layer[i].getBandwidth(newLevelP->fps_scale * ti.start_fps);
    level_bw += layer_bw[i];
    //cout << "Layer" << i << " : " << layer_bw[i] << endl;
  }

  // ===== remove layers of the old level from priority classes =====================
  
  card8  oldPrio;
  card64 bw;

  if( oldLevel != 255 ) {                                 // is there a old level ?
    for(i=0; i < (int) oldLevelP->layer.size(); i++) {
      oldPrio = oldLevelP->layer[i].priorityClass;
      bw      = oldLevelP->layer[i].getBandwidth(oldLevelP->fps_scale * ti.start_fps);
      qm->freeInPrio[ oldPrio ] += bw;
    }
  }

  //  for(i=0; i < 16; i++) {
  //    cout << qm->freeInPrio[i] << endl;
  //  }


  // ===== search a priority class for each layer of the new level ==================
  
  for(i=0; i < numLayers; i++) {                        // for each layer ...
    for(j=qm->numPrios-1; j >= 0; j--) {                // search a class.
      if( layerFitsInClass(j, layer_bw[i], newLevelP->layer[i].maxLossRate, newLevelP->MaxTransferDelay ) ) {
	//if( qm->freeInPrio[j] >= layer_bw[i] ) {          // fits layer i in class j ?
	layerFit[i] = true;                             // layer i fits !
	qm -> freeInPrio[j] -= layer_bw[i];             // allocate bandwidth
	newLevelP->layer[i].priorityClass = j;          // set layers priority class
	break;                                          // class found for layer i ! So break!
      }
    }
    if(!layerFit[i]) {                                  // no class found for layer i ?
      break;                                            // stop searching!
    }
  }

  // ===== test, whether all layers had found a class ==================================

  bool fit = true;
  
  for(i=0; i < numLayers; i++) {
    if(layerFit[i] == false) {
      fit = false;
      break;
    }
  }


  // ===== remove new layers and add old layers, if not all new layers fit =============

  card64 old_bw;

  if(!fit) {
    // remove new layers from priority classes
    for(i=0; i < numLayers; i++) {
      if(layerFit[i]) {
	oldPrio = newLevelP->layer[i].priorityClass;
	qm -> freeInPrio[ oldPrio ] += layer_bw[i];
      } else {
	break;
      }
    }

    // add old layers to priority classes
    if( oldLevel != 255 ) {
      for(i=0; i < (int) oldLevelP->layer.size() ; i++ ) {
	oldPrio = oldLevelP->layer[i].priorityClass;
	old_bw = oldLevelP->layer[i].getBandwidth( oldLevelP->fps_scale * ti.start_fps );
	qm->freeInPrio[ oldPrio ] -= old_bw;
      }
    }
    return false;
  } else {                                    // all layers fit
    level = newLevel;
  }

  // cout << "new level: " << (int) newLevel << endl;

  /*
  for(i=0; i < numLayers; i++) {
    cout << "Priority class of layer " << i << " : " << (int) newLevelP -> layer[i].priorityClass << endl;
  }
  */
  current_bw = level_bw;                       // set the new bandwidth of the flow
  return true;
}

bool Flow::scaleDown(card32 sessionID)
{
  // cout << "scale down flow " << flowID << " of session " << sessionID << endl;

  int i;

  if(level == 255) return false;

  //  card8 oldLevel = level;
  Level* oldLevelP = &(ti.QualityLevel[level]);
  card8 newLevel = oldLevelP->LevelDown;
  Level* newLevelP;

  if(newLevel != 255) {
    newLevelP = &(ti.QualityLevel[newLevel]);
    if(newLevelP->layer.size() > oldLevelP->layer.size()) {
      cout << "Error: Scale down: Number of layers must not grow, when scale down !!!" << endl;
      return false;
    }
  } else {                          // new level = 255 ?
    return false;
  }

  // ===== remove old layers ============================================================
  vector<card8> prioClass((int)oldLevelP->layer.size());
  vector<card64> old_bw((int)oldLevelP->layer.size());

  for(i=0; i < (int) oldLevelP->layer.size(); i++ ) {
    prioClass[i] = oldLevelP->layer[i].priorityClass;
    old_bw[i]        = oldLevelP->layer[i].getBandwidth(oldLevelP->fps_scale * ti.start_fps);
    qm->freeInPrio[ prioClass[i] ] += old_bw[i];
  }

  // ===== add new layers ===============================================================
  card64 new_bw;
  card64 level_bw = 0;

  if(newLevel != 255) {
    for(i=0; i < (int) newLevelP->layer.size(); i++ ) {
      new_bw =  newLevelP->layer[i].getBandwidth(newLevelP->fps_scale * ti.start_fps);
      if(new_bw > old_bw[i]) {
	cout << "Error: Scale down: Layer must not became bigger when scale down !!!" << endl;
	return false;
      }
      qm->freeInPrio[ prioClass[i] ] -= new_bw;
      newLevelP->layer[i].priorityClass = prioClass[i];     // keep priority classes of the old level
      level_bw += new_bw;
    }
  }

  current_bw = level_bw;

  level = newLevel;  
  /*
  for(int i=0; i < 16; i++) {
    cout << "Priority Class " << i << " :  "<< qm->freeInPrio[i] << endl;
  }
  */
  // cout << "new level " << (int) level << endl;

  return true;
}

bool Flow::layerFitsInClass(card8 priorityClass, card64 bandwidth, card8 lossRate, card32 delay)
{
  if(bandwidth > qm->freeInPrio[priorityClass] ) return false;
  if(lossRate < lossRateOfClass[ priorityClass ]) return false;
  if(delay < delayOfClass[ priorityClass ]) return false;
  return true;
}

}
