// ##########################################################################
// ####                                                                  ####
// ####                      RTP Audio Server Project                    ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### Flow Info                                                        ####
// ####                                                                  ####
// #### Version 1.00, February 17, 2000                                  ####
// ####                                                                  ####
// #### Copyright (C) 1999  Thomas Dreibholz                             ####
// ####               2000  Universitt Bonn, Abt. IV                    ####
// ####                     EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://titan.cs.uni-bonn.de/~dreibhol ####
// ####                                                                  ####
// ##########################################################################


#ifndef FLOWINFO_H
#define FLOWINFO_H


#include "../system.h"
#include "../socketaddress.h"


namespace Coral {


/** 
  * This class inherits SocketAddress and contains an additional flow label
  * for IPv6 support.
  *
  * @short   Flow Info
  * @author  Thomas Dreibholz (Dreibholz@bigfoot.com)
  * @version 1.0
  */            
class FlowInfo : public SocketAddress
{
   // ====== Constructors/Destructor ========================================
   public:
   /**
     * Constructor for a new FlowInfo.
     *
     */
   FlowInfo();

   /**
     * Constructor for a new FlowInfo.
     *
     * @param destination Destination address.
     * @param flowLabel Flow label (20 bits).
     * @param trafficClass Traffic class (8 bits).
     */
   FlowInfo(const SocketAddress* destination,
            const card32         flowLabel,
            const card8          trafficClass);


   // ====== Initialization =================================================
   /**
     * Initialize the flow info from the system's sockaddr structure. The
     * sockaddr structure may be sockaddr_in (IPv4) or sockaddr_in6 (IPv6).
     *
     * @address sockaddr.
     * @length Length of sockaddr (sizeof(sockaddr_in) or sizeof(sockaddr_in6)).
     * @flowInfo Flow info.
     */
   void init(sockaddr* address, const card32 length, const card32 flowInfo);


   // ====== Status functions ===============================================
   /**
     * Get IPv6 flow info: (flowLabel | (trafficClass << 20)).
     *
     * @return Flow info.
     */
   inline card32 getFlowInfo() const;

   /**
     * Get flow label.
     *
     * @return Flow label.
     */
   inline card32 getFlowLabel() const;

   /**
     * Get traffic class.
     *
     * @return Traffic class.
     */
   inline card8 getTrafficClass() const;


   /**
     * Set traffic class.
     *
     * @param trafficClass New traffic class.
     */
   inline void setTrafficClass(const card8 trafficClass);


   // ====== Private data ===================================================
   private:
   card32 FlowInfo;
};


#include "flowinfo.icc"


}


#endif
