//########################################################
//##                                                    ##
//##                 QoS Management                     ##
//##                 ==============                     ##
//##                                                    ##
//## Layer                                              ##
//##                                                    ##
//## Version 1.0                                        ##
//## Date  : March 2000                                 ##
//## Author: Simon Vey (vey@cs.uni-bonn.de)             ##
//##                                                    ##
//########################################################

#ifndef LAYER_H
#define LAYER_H

#include "../system.h"

namespace Coral {

class Level;

/**
 * This class describes one layer. Every level consists of several layers.
 *
 * @short Layer
 * @author Simon Vey
 * @version 1.0
 */
class Layer {
 public:
  /**
   * Constructor.
   */
  Layer(Level* level_);

  /**
   * Calculates the bandwidth needed by this layer, when sending with a given
   * frames-per-second value.
   *
   * @param fps frames per second, with wich the layer will be sent.
   * @return bandwidth, needed by the layer 
   */

  card8     findClass();

  card32    getBandwidth();

  void      setFrameSize(card32 size);
  card32    getFrameSize();

  void      setLossRate(card8 lossRate);
  card8     getLossRate();

  void      setPriorityClass(card8 class_);
  card8     getPriorityClass();

  void      setTrafficClass(card8 class_);
  card8     getTrafficClass();

  void      setLevel(Level* level_);

 private:
  card32 frameSize;

  card8  maxLossRate;

  card8  priorityClass;

  card8  trafficClass;
  
  Level* level;
};

}

#include "layer.icc"

#endif
