#ifndef LAYEREDSTREAM_H
#define LAYEREDSTREAM_H

#include "level.h"
#include "qosmanagerinterface.h"
#include "../extendedtransportinfo.h"
#include "../system.h"
#include <vector>

namespace Coral {

class Session;

class LayeredStream {
 public:
  LayeredStream(card32 ID, ExtendedTransportInfo ti, QoSManagerInterface* qmi, Session* session_);

  ~LayeredStream();

  //void      correctPointer();
  card8     getQuality();

  void      setID(card32 ID);
  card32    getID();

  void      setLevel(card8 cLevel);
  card8     getLevel();

  void      setWantedBandwidth(card64 bw);
  card64    getWantedBandwidth();

  void      setCurrentBandwidth(card64 bw);
  card64    getCurrentBandwidth();

  void      setStartFPS(card32 fps);
  card32    getStartFPS();

  void      setLastChanged(card64 time);
  card64    getLastChanged();

  void      addLevel(Level* lev);
  Level*    getLevel(int index); 

  QoSManagerInterface* getInterface();

  Session*  getSession();
  void      setSession(Session* s);

  bool      scaleDown();
  bool      scaleUp();

  void      rememberOldLevel();
  bool      resetToOldLevel();

  card64    getBandwidth();

  ExtendedTransportInfo getTransportInfo();

  card32    getNumLevels();

  void      setMaximalLevel(card32 lev);
  card32    getMaximalLevel();

 private:
  Session* session;

  ExtendedTransportInfo ti_alt;

  card32 id;

  card8 currentLevel;

  card64 wantedBandwidth;

  card64 currentBandwidth;

  card32 start_fps;

  card64 lastChanged;                 // in mikroseconds

  int  maxLevel; 

  QoSManagerInterface* interface;

  vector<Level*> level;

  card8 levelToRemember;
  
};

}

#include "layeredstream.icc"

#endif
