#include "level.h"
#include "layeredstream.h"
#include "session.h"
#include "qosmanager.h"

namespace Coral {

Level::Level(LayeredStream* stream_)
  : stream(stream_)
{
  // cout << "levelconstructor: stream: " << stream << endl;
}

Level::~Level()
{
  int i=0;
  for(i=0; i < (int)layer.size(); i++) {
    delete(layer[i]);
  }
}

card32 Level::getFPS()
{
  return (card32) (stream->getStartFPS() * fps_scale);
}

bool Level::findClasses()
{
  int i=0;

  //cout << "streamID " << stream->getID() << endl;
  
  Session*    session = stream -> getSession();
 
  //cout << "sessionID:" << session->getID()  << endl;

  QoSManager* qm      = session -> getQoSManager();

  //cout << "2" << endl;

  vector<bool>  layerFits(layer.size());  

  /*
  vector<card8> oldClass(layer.size());

  for(i=0; i < layer.size(); i++) {
    oldClass[i] = layer[i].getPriorityClass();
  }
  */

  bool allLayersFit = true;

  //cout << "3" << endl;

  for(i=0; i < (int)layer.size(); i++) {
    layerFits[i] = false;
    if(layer[i]->findClass() != 255) {
      //cout << "layer " << i << " fits" << endl;
      layerFits[i] = true;
    } else {
      //cout << "layer " << i << " does not fit" << endl;
      allLayersFit = false;
      break;
    }
  }

  //cout << "4" << endl;

  if(!allLayersFit) {
    for(i=0; i < (int)layer.size(); i++) {
      if(layerFits[i]) {
	qm->setFreeBandwidthOfClass(layer[i]->getPriorityClass(),
                                    qm->getFreeBandwidthOfClass(layer[i]->getPriorityClass()) + layer[i]->getBandwidth());
	layer[i]->setPriorityClass(255);
      } else {
	break;
      }
    }
    return false;
  } else {
    return true;
  }
}

bool Level::classOk(card8 class_)
{
  int i=0;
  
  Session* session = stream->getSession();

  if(maxTransferDelay < session->getDelayOfClass(class_) ) {
    return false;
  } 

  for(i=0; i < (int) layer.size(); i++) {
    if(layer[i]->getLossRate() < session->getLossRateOfClass(class_)) {
      return false;
    }
  }
  
  return true;
}

card64 Level::getBandwidth()
{
  int i=0;
  card64 result=0;
  for(i=0; i < (int) layer.size(); i++) {
    result += layer[i]->getBandwidth();
  }
  return result;
}

}
