#include "session.h"
#include "qosmanager.h"

namespace Coral {

Session::Session(QoSManager* qm_, PortableAddress dest)
  : qm(qm_), destination(dest)
{
  int i=0;
  for(i=0; i < (int) qm->getNumPrios(); i++ ) {
    delayOfClass[i] = 0;
    lossRateOfClass[i] = 0;
  }
}

QoSManager* Session::getQoSManager()
{
  //cout << "qm " << qm << endl;
  return qm;
}


card32 Session::newStream(card32 ID, ExtendedTransportInfo ti, QoSManagerInterface* interface)
{
  card32 streamIndex;

  LayeredStream* s=new LayeredStream(ID,ti,interface,this);

  stream.push_back(s);

  streamIndex = stream.size()-1;

  return ID;
}

bool Session::closeStream(card32 ID)
{
  int i,j;
  Level* levelP ;
  card8 prioClass;
  card64 bw;

  vector<LayeredStream*>::iterator iter = stream.begin();

  for(i=0; i < (int) stream.size(); i++) {
    if(stream[i]->getID() == ID) {

      // ===== free bandwidth in priority classes ============================================
      if(stream[i]->getLevel() != 255) {
	levelP = stream[i]->getLevel( stream[i]->getLevel() );

	for(j=0; j < (int) levelP->getNumLayers(); j++) {
	  prioClass = levelP->getLayer(j)->getPriorityClass();
	  bw        = levelP->getLayer(j)->getBandwidth();
	  qm->setFreeBandwidthOfClass(prioClass, qm->getFreeBandwidthOfClass(prioClass) + bw);
	}
      }

      // ===== remove flow from session =====================================================
      delete(stream[i]);
      stream.erase(iter);


      return true;
    }
    iter++;
  }

  // for(i=0; i< 16; i++) {
  //   cout << "free in class " << i << " : " << qm->getFreeBandwidthOfClass(i) << endl;
  // }

  return false;
} 


card32 Session::updateDelayOfClass(card8 class_, card32 measuredDelay)
{
  delayOfClass[class_] = delayOfClass[class_] / 2 + measuredDelay / 2;
  return delayOfClass[class_];
}

card8 Session::updateLossRateOfClass(card8 class_, card8 lossRate)
{
  lossRateOfClass[class_] = lossRateOfClass[class_] / 2 + lossRate / 2;

  //cout << "lossrate estimation of class " << (int) class_ << " : " << (int) lossRateOfClass[class_] << endl;

  return lossRateOfClass[class_];
}

/*
bool Session::scaleDown(bool force)
{
  int i;

  // search a flow to scale down
  long int maximum = 0;
  int index = 0;
  long int tooMuch;

  if(flow.size() > 0) 
    maximum = flow[0].current_bw - flow[0].bandwidthCalculation;

  for(i=0; i < (int) flow.size(); i++) {
    tooMuch = flow[i].current_bw - flow[i].bandwidthCalculation;
    if(tooMuch > maximum) {
      index = i;
      maximum = tooMuch;
    }
  }

  if(force || (maximum > 0)) {
    return flow[index].scaleDown(sessionID);    
  } else {
    return false;
  } 
}
*/

/*
card64 Session::getBandwidth()
{
  int i;
  card64 result = 0;

  for(i=0; i < (int) stream.size(); i++) {
    result += stream[i].getBandwidth();
  }

  return result;
}
*/

/*
card32 Session::getNumStreams() const
{
  return stream.size();
}

void Session::setID(card32 ID)
{
  id = ID;
}

card32 Session::getID()
{
  return id;
}

void Session::setWeight(card32 weight_)
{
  weight = weight_;
}

card32 Session::getWeight()
{
  return weight;
}

void Session::addStream(LayeredStream str)
{
  stream.push_back(str);
}

void Session::setDelayOfClass(card8 class_, card32 delay)
{
  delayOfClass[class_] = delay;
}

card32 Session::getDelayOfClass(card8 class_)
{
  return delayOfClass[class_];
}

void Session::setLossRateOfClass(card8 class_, card32 lossRate) 
{
  lossRateOfClass[class_] = lossRate;
}

card32 Session::getLossRateOfClass(card8 class_)
{
  return lossRateOfClass[class_];
}

LayeredStream* Session::getStream(int index)
{
  return &stream[index];
}
*/
}
