// ##########################################################################
// ####                                                                  ####
// ####                      RTP Audio Server Project                    ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### Audio Decoder Interface                                          ####
// ####                                                                  ####
// #### Version 1.00  --  February 23, 2001                              ####
// ####                                                                  ####
// #### Copyright (C) 1999  Thomas Dreibholz                             ####
// ####               2000  Universitt Bonn, Abt. IV                    ####
// ####               2001  EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################


#ifndef AUDIODECODERINTERFACE_H
#define AUDIODECODERINTERFACE_H


#include "system.h"
#include "decoderinterface.h"
#include "audioquality.h"


namespace Coral {


/**
  * This class is the interface for an audio decoder.
  *
  * @short   Audio Decoder Interface
  * @author  Thomas Dreibholz (Dreibholz@bigfoot.com)
  * @version 1.0
  */
class AudioDecoderInterface : virtual public DecoderInterface,
                              virtual public AudioQualityInterface
{
   // ====== Wanted quality =================================================
   public:
   /**
     * Get wanted quality. This is the quality wanted in TransportInfo
     * from getTransportInfo().
     *
     * @return Wanted quality.
     *
     * @see DecoderInterface#getTransportInfo
     */
   virtual AudioQuality getWantedQuality() const = 0;

   /**
     * Set wanted quality. This is the quality wanted in TransportInfo
     * from getTransportInfo(). Note: This does *not* tell the sender to
     * modify the quality! This function only sets the wanted quality which
     * is reported by getTransportInfo().
     *
     * @return Wanted quality.
     *
     * @see DecoderInterface#getTransportInfo
     */
   virtual void setWantedQuality(const AudioQualityInterface& wantedQuality) = 0;
};


}


#endif
  