// ##########################################################################
// ####                                                                  ####
// ####                      RTP Audio Server Project                    ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### Extended Transport Info                                          ####
// ####                                                                  ####
// #### Version 1.00  --  February 16, 2001                              ####
// ####                                                                  ####
// #### Copyright (C) 1999  Thomas Dreibholz                             ####
// ####               2000  Universitt Bonn, Abt. IV                    ####
// ####               2001  EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################


#ifndef EXTENDEDTRANSPORTINFO_H
#define EXTENDEDTRANSPORTINFO_H


#include "system.h"
#include "range.h"
#include "transportinfo.h"
#include "streamsrcdest.h"


namespace Coral {


/**
  * This class is an extension to TransportInfo and contains additional
  * information on source and destination.
  *
  * @short   Extended Transport Information
  * @author  Thomas Dreibholz (Dreibholz@bigfoot.com)
  * @version 1.0
  */
class ExtendedTransportInfo : public TransportInfo
{
   // ====== Constructor ====================================================
   public:
   /**
     * Constructor.
     */
   ExtendedTransportInfo();


   // ====== Byte order translation =========================================
   /**
     * Translate byte order.
     */
   void translate();


   // ====== Reset ==========================================================
   /**
     * Reset.
     */
   void reset();


   // ====== Comparision operators ==========================================
   /**
     * == operator.
     */
   int operator==(const ExtendedTransportInfo& eti) const;

   /**
     * != operator.
     */
   inline int operator!=(const ExtendedTransportInfo& eti) const;


   // ====== Values =========================================================
   public:
   /**
     * Source and destination addresses incl. traffic class and flow label.
     */
   StreamSrcDest SrcDest[TransportInfoLevel::MaxQualityLayers];
};


/**
  * << operator.
  */
ostream& operator<<(ostream& os, const ExtendedTransportInfo& eti);


}


#include "extendedtransportinfo.icc"


#endif
