// ##########################################################################
// ####                                                                  ####
// ####                      RTP Audio Server Project                    ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### Internet Flow Implementation                                     ####
// ####                                                                  ####
// #### Version 1.00  --  February 16, 2001                              ####
// ####                                                                  ####
// #### Copyright (C) 1999  Thomas Dreibholz                             ####
// ####               2000  Universitt Bonn, Abt. IV                    ####
// ####               2001  EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################


#include "system.h"
#include "internetaddress.h"
#include "internetflow.h"


namespace Coral {


// ###### Constructor #######################################################
InternetFlow::InternetFlow()
   : InternetAddress()
{
   FlowInfo = 0;
}


// ###### Constructor #######################################################
InternetFlow::InternetFlow(const InternetFlow& flow)
   : InternetAddress(flow)
{
   FlowInfo = flow.FlowInfo;
}


// ###### Constructor #######################################################
InternetFlow::InternetFlow(const InternetAddress& address,
                           const card32           flowLabel,
                           const card8            trafficClass)
   : InternetAddress(address)
{
   FlowInfo = htonl(flowLabel | ((card32)trafficClass << 20));
}


// ###### Reset #############################################################
void InternetFlow::reset()
{
   InternetAddress::reset();
   FlowInfo = 0;
}


// ###### Get system address ################################################
cardinal InternetFlow::getSystemAddress(sockaddr*       buffer,
                                        const socklen_t length,
                                        const cardinal  type) const
{
   const cardinal addressLength = InternetAddress::getSystemAddress(buffer,length,type);
   if((addressLength > 0) && (type == AF_INET6)) {
      sockaddr_in6* address  = (sockaddr_in6*)buffer;
      address->sin6_flowinfo = FlowInfo;
   }
   return(addressLength);
}


// ###### Set system address ################################################
bool InternetFlow::setSystemAddress(sockaddr* address, const socklen_t length)
{
   FlowInfo = 0;
   if(InternetAddress::setSystemAddress(address,length)) {
      sockaddr_in6* address6 = (sockaddr_in6*)address;
      if(address6->sin6_family == AF_INET6) {
         FlowInfo = address6->sin6_flowinfo;
      }
      return(true);
   }
   return(false);
}


// ###### Get address string ################################################
String InternetFlow::getAddressString() const
{
   String result = InternetAddress::getAddressString();
   char str[32];
   snprintf((char*)&str,sizeof(str),
            "/$%05x, $%02x",getFlowLabel(),getTrafficClass());
   return(result + (char*)&str);
}


}
