// ##########################################################################
// ####                                                                  ####
// ####                      RTP Audio Server Project                    ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### Congestion Monitor                                               ####
// ####                                                                  ####
// #### Version 1.00  --  February 23, 2001                              ####
// ####                                                                  ####
// #### Copyright (C) 1999  Thomas Dreibholz                             ####
// ####               2000  Universitt Bonn, Abt. IV                    ####
// ####               2001  EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################


#include "system.h"
#include "networkmonitorinterface.h"
#include "networkmonitor.h"
#include "congestionmonitor.h"


#include <signal.h>


using namespace Coral;


// ###### Main program ######################################################
int main(int argc, char** argv)
{
   char*  device    = NULL;
   char*  local     = NULL;
   card64 interval  = 1000000;
   card64 bandwidth = 0;
   card16 port      = 0;

   // ====== Get arguments ==================================================
   if(argc < 2) {
      cerr << "Usage: " << argv[0] << endl
           << "       [Port] {-device=name} {-bandwidth=bit/s} {-interval=ms} {-local=hostname}"
           << endl;
      exit(1);
   }
   for(cardinal i = 2;i < (cardinal)argc;i++) {
      if(!(strncasecmp(argv[i],"-device=",8))) {
          device = &argv[i][8];
      }
      else if(!(strncasecmp(argv[i],"-bandwidth=",11))) {
          bandwidth = atol(&argv[i][11]);
      }
      else if(!(strncasecmp(argv[i],"-interval=",10))) {
         interval = (card64)atol(&argv[i][10]) * 1000000;
      }
      else if(!(strncasecmp(argv[i],"-local=",7))) {
         local = &argv[i][7];
      }
      else {
         cerr << "Wrong parameter: " << argv[i] << endl;
         cerr << "Usage: " << argv[0] << endl
              << "       [Port] {-device=name} {-bandwidth=bit/s} {-interval=ms} {-local=hostname}"
              << endl;
         exit(1);
      } 
   }
   port = atoi(argv[1]);
   if(device == NULL) {
      cerr << "ERROR: No device given (-device=name)!" << endl;
      exit(1);
   }
   if(bandwidth < 1000) {
      cerr << "WARNING: No bandwidth given. Using default 10 MBit/s!" << endl;
      bandwidth = 10000000;
   }
   if(interval < 1000)
      interval = 1000;
   if(interval > 60000000)
      interval = 60000000;


   // ====== Creating connection to reportReceiver ==========================
   InternetAddress monitorAddress(local,port);
   Socket          monitorSocket(Socket::IP,Socket::TCP);
   monitorSocket.setSoReuseAddress(true);
   monitorSocket.setSoLinger(true,3);
   if(monitorSocket.bind(monitorAddress) == false) {
      cerr << "ERROR: Bind to " << monitorAddress << " failed!" << endl;
      exit(1);
   }
   monitorSocket.listen(5);

   // ====== Start congestion monitor =======================================
   NetworkMonitor* netMonitor = new NetworkMonitor(NULL,device);
   if((netMonitor != NULL) && (netMonitor->ready())) {
      CongestionMonitor* cgMonitor = new CongestionMonitor(netMonitor,bandwidth / 8);
      if(cgMonitor != NULL) {
         cgMonitor->setInterval(interval);
         netMonitor->start();
         cgMonitor->start();

         // ====== Wait for break ==============================================
         // SIGPIPE has to be ignored!! It is sent, if a TCP connection is
         // closed and a write() is tried -> Program terminates with SIGPIPE.
         // But we want the monitor only to remove the receiver. Therefore:
         signal(SIGPIPE,SIG_IGN);

         // ====== Print information ===========================================
         cout << "Congestion Monitor - Copyright (C) 1999-2001 Thomas Dreibholz" << endl;
         cout << "-------------------------------------------------------------" << endl;
         cout << endl;
         cout << "Monitor address: " << monitorAddress << endl;
         cout << "Device:          " << device << endl;
         cout << "Bandwidth:       " << bandwidth << endl;
         cout << "Interval:        " << interval << endl;
         cout << endl;
    
         // ====== Main loop ===================================================
         for(;;) {
            Socket* receiver = monitorSocket.accept();
            if(receiver != NULL) {
               cgMonitor->addReportReceiver(receiver);
            }
         }

         // ====== Clean-up ====================================================
         cgMonitor->stop();
         delete cgMonitor;
      }
      netMonitor->stop();
      delete netMonitor;
   }
   return(0);
}
