// ##########################################################################
// ####                                                                  ####
// ####                      RTP Audio Server Project                    ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### NetLog Reader Implementation                                     ####
// ####                                                                  ####
// #### Version 1.00  --  February 23, 2001                              ####
// ####                                                                  ####
// #### Copyright (C) 1999  Thomas Dreibholz                             ####
// ####               2000  Universitt Bonn, Abt. IV                    ####
// ####               2001  EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################


#include "system.h"
#include "netlogreader.h"
#include "timedthread.h"
#include "networkmonitorinterface.h"
#include "streammonitorreport.h"
#include "netlogfileheader.h"
#include "tools.h"


namespace Coral {


// ###### Constructor #######################################################
NetLogReader::NetLogReader(const char* name)
{
   // ====== Initialization =================================================
   TimeStamp      = 0;
   Interval       = 0;
   FileDescriptor = fopen(name,"r");
   if(FileDescriptor == NULL) {
      cerr << "ERROR: Unable to read log file <" << name << ">!" << endl;
      return;
   }

   // ====== Read header from file ==========================================
   NetLogFileHeader nlfh;
   if(fread((void *)&nlfh,sizeof(nlfh),1,FileDescriptor) != 1) {
      cerr << "ERROR: Read error!" << endl;
      fclose(FileDescriptor);
      FileDescriptor = NULL;
      return;
   }

   // ====== Check header ===================================================
   if( (strncmp((char*)&nlfh.ID,NetLogFileHeaderID,4)) ||
       (translate16(nlfh.Version) != NetLogFileHeaderVersion) ||
       (translate16(nlfh.EntrySize) != sizeof(NetworkMonitorReport)) ) {
       cerr << "ERROR: File <" << name << "> has invalid format!" << endl;
      fclose(FileDescriptor);
      FileDescriptor = NULL;
      return;       
   }

   TimeStamp         = translate64(nlfh.TimeStamp);
   Interval          = translate64(nlfh.Interval);
   Flags             = translate32(nlfh.Flags);
   StreamReportCount = 0;
}


// ###### Read next report from file ########################################
bool NetLogReader::readNextReport(NetworkMonitorReport* report)
{
   while(StreamReportCount > 0) {
      StreamReport sr;
      readNextStreamReport(&sr);
   }
   if(FileDescriptor != NULL) {
      if(fread((void*)report,sizeof(NetworkMonitorReport),1,FileDescriptor) == 1) {
         report->translate();
         if(Flags & NLFH_ContainsStreams) {
            if(fread((void*)&StreamReportCount,sizeof(card32),1,FileDescriptor) == 1) {
               StreamReportCount = translate32(StreamReportCount);
            }
         }
         return(true);
      }
   }
   return(false);
}


// ###### Read next report from file ########################################
bool NetLogReader::readNextStreamReport(StreamReport* report)
{
   if(FileDescriptor != NULL) {
      if(StreamReportCount > 0) {
         if(fread((void*)report,sizeof(StreamReport),1,FileDescriptor) == 1) {
            report->translate();
            StreamReportCount--;
            return(true);
         }
      }
   }
   return(false);
}


// ###### Destructor ########################################################
NetLogReader::~NetLogReader()
{
   if(FileDescriptor) {
      fclose(FileDescriptor);
      FileDescriptor = NULL;
   }
}


}
