// ##########################################################################
// ####                                                                  ####
// ####                      RTP Audio Server Project                    ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### Network Monitor Report Implementation                            ####
// ####                                                                  ####
// #### Version 1.00  --  February 23, 2001                              ####
// ####                                                                  ####
// #### Copyright (C) 1999  Thomas Dreibholz                             ####
// ####               2000  Universitt Bonn, Abt. IV                    ####
// ####               2001  EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################


#include "system.h"
#include "networkmonitorreport.h"
#include "tools.h"


namespace Coral {


// ###### Constructor #######################################################
NetworkMonitorReport::NetworkMonitorReport()
{
   reset();
}


// ###### Reset #############################################################
void NetworkMonitorReport::reset()
{
   Interval     = 0;
   TimeStamp    = 0;

   BytesRaw     = 0;
   BytesIPv4    = 0;
   BytesIPv6    = 0;
   BytesTCP     = 0;
   BytesTCPData = 0;
   BytesUDP     = 0;
   BytesUDPData = 0;
   BytesICMP4   = 0;
   BytesICMP6   = 0;

   PacketsRaw   = 0;
   PacketsIPv4  = 0;
   PacketsIPv6  = 0;
   PacketsTCP   = 0;
   PacketsUDP   = 0;
   PacketsICMP4 = 0;
   PacketsICMP6 = 0;

   for(cardinal i = 0;i < TrafficClassValues::MaxValues;i++) {
      ClassBytes[i]   = 0;
      ClassPackets[i] = 0;
   }
}


// ###### Translate byte order ##############################################
void NetworkMonitorReport::translate()
{
   Interval     = translate64(Interval);
   TimeStamp    = translate64(TimeStamp);
   BytesRaw     = translate64(BytesRaw);
   BytesIPv4    = translate64(BytesIPv4);
   BytesIPv6    = translate64(BytesIPv6);
   BytesTCP     = translate64(BytesTCP);
   BytesTCPData = translate64(BytesTCPData);
   BytesUDP     = translate64(BytesUDP);
   BytesUDPData = translate64(BytesUDPData);
   BytesICMP4   = translate64(BytesICMP4);
   BytesICMP6   = translate64(BytesICMP6);
   PacketsRaw   = translate32(PacketsRaw);
   PacketsIPv4  = translate32(PacketsIPv4);
   PacketsIPv6  = translate32(PacketsIPv6);
   PacketsTCP   = translate32(PacketsTCP);
   PacketsUDP   = translate32(PacketsUDP);
   PacketsICMP4 = translate32(PacketsICMP4);
   PacketsICMP6 = translate32(PacketsICMP6);
   for(cardinal i = 0;i < TrafficClassValues::MaxValues;i++) {
      ClassBytes[i]   = translate64(ClassBytes[i]);
      ClassPackets[i] = translate32(ClassPackets[i]);
   }
}


// ###### << operator #######################################################
ostream& operator<<(ostream& os, const NetworkMonitorReport& report)
{
   char buffer[1024];
   snprintf((char*)&buffer,sizeof(buffer),
          "iv=%Ld ts=%Ld raw=%d@%LdB tcp=%LdB (%d@%LdB) udp=%Ld (%d@%LdB) ip4=%d@%LdB ip6=%d@%LdB icmp=%d@%LdB icmp6=%d@%LdB",
          report.Interval,
          report.TimeStamp,
          report.PacketsRaw,report.BytesRaw,
          report.BytesTCPData,report.PacketsTCP,report.BytesTCP,
          report.BytesUDPData,report.PacketsUDP,report.BytesUDP,
          report.PacketsIPv4,report.BytesIPv4,
          report.PacketsIPv6,report.BytesIPv6,
          report.PacketsICMP4,report.BytesICMP4,
          report.PacketsICMP6,report.BytesICMP6);
   os << buffer;
   return(os);
}


}
