package bonn.infoIV.v3d2.rtp;

public class Control
{
  public static final int OWN_SSRC = 872659128;
  public static final int V3D2     = 0x56334432;  // V = 86, 3 = 51, D = 68, 2 = 50

  public static final boolean NO_USE = false;

  protected int[] FramesPerSec_       = null; // Application set
  protected int[] BitsPerSec_         = null; // Compute: App-requiered ImageQuality ~ Set FramesPerSec_ [FEEDBACK???]
  protected int[] SplitsPercentage_   = null;
  protected int[] StarterPacket_      = null;
  protected float FractionLost_       = 0.0f; // Computed with DataChannel-variables
  public    int   PacketsLost_        =  0;   // Computed by DataChannel
  public    int   HighestSeqNum_      =  0;   // Computed by DataChannel
  protected int   InterarrivalJitter_ = 10;   // NOT supported yet
  protected int   Lsr_                = 20;   // NOT supported yet
  protected int   Dlsr_               = 30;   // NOT supported yet


  private RTCPPacket      AppPacket_ = null;
  private RTCPPacket      RepPacket_ = null;

  private ControlChannel MyChannel_ = null;

  public Control(ControlChannel channel)
  {
    super();
    MyChannel_ = channel;
    AppPacket_ = new RTCPPacket(false, OWN_SSRC, V3D2);
    RepPacket_ = new RTCPPacket(false, 1, OWN_SSRC, new RTCPReportBlock());

    FramesPerSec_     = new int[1];
    BitsPerSec_       = new int[1];
    SplitsPercentage_ = new int[1];
    StarterPacket_    = new int[4];
 
  }

  public void updateFps(int fps)
  {
    FramesPerSec_[0] = fps;
    AppPacket_.MyHeader_.setSubtype(PayloadType.FRAMES_PER_SEC);
    AppPacket_.setApplicationData(FramesPerSec_);
    MyChannel_.send(AppPacket_);
  }

  public void updateBps(int bps)
  {
    BitsPerSec_[0] = bps;
    AppPacket_.MyHeader_.setSubtype(PayloadType.BITS_PER_SEC);
    AppPacket_.setApplicationData(BitsPerSec_);
    MyChannel_.send(AppPacket_);
  }

  public void updateSplitsPercentage(int percentage)
  {
    SplitsPercentage_[0] = percentage;
    AppPacket_.MyHeader_.setSubtype(PayloadType.SPLITS_PERCENTAGE);
    AppPacket_.setApplicationData(SplitsPercentage_);
    MyChannel_.send(AppPacket_);
  }

  public void startTransfer(int loop, int[] data)
  {
    if (data.length<3)
    {
      System.err.println("[ERROR]: transfer could not be initiated, cause not all data available");
      return;
    }
    StarterPacket_[0] = loop;
    StarterPacket_[1] = data[0];
    StarterPacket_[2] = data[1];
    StarterPacket_[3] = data[2];

    AppPacket_.MyHeader_.setSubtype(PayloadType.START_TRANSFER);
    AppPacket_.setApplicationData(StarterPacket_);

    MyChannel_.send(AppPacket_);
  }

  public void sendReceiverReport(int ssrc, int lostInterval, int expectedInterval)
  {
    //byte test = 127;

    if(expectedInterval == 0) return;
    if(lostInterval <= 0) FractionLost_ = 0f;
    else FractionLost_ = (lostInterval<<8)/expectedInterval;

    RepPacket_.ReportBlocks_[0].setSSRC(ssrc);
    RepPacket_.ReportBlocks_[0].setLostFraction((byte)FractionLost_);
    RepPacket_.ReportBlocks_[0].setLostPackets(PacketsLost_);
    RepPacket_.ReportBlocks_[0].setHighestSeqNumber(HighestSeqNum_);

    MyChannel_.send(RepPacket_);
  }
}

