/**
 * MPI-Graphics package: originaly designed and implemented by: 
 *
 * @author Robert Schneider of MPI-Saarbruecken
 *
 * further developed and changed by:
 * @author Stavros Pechlivanidis of Universitaet Bonn, Abt.IV 
 * @author Thomas Dreibholz of Universitaet Bonn, Abt.IV
 */
package mpi_graphics;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import com.sun.j3d.utils.universe.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import java.io.*;
import com.sun.j3d.utils.behaviors.mouse.*;
import java.util.*;

public class Animation_Container extends OFF_Container 
{
  public static final int PROZENT_INIT = 100; // initial value of percentage of desired quality
  public static final int FPS_INIT     = 1;   // initial value of desired Frames per second
  public static final int MBPS_INIT    = 2;   // initial value of desired transmission Rate (Mbit per sec)

  private Canvas3D_MPI_Renderer Canvas_ = null;
  private Callback_PM MyAnimCB_         = null;
  private JCheckBox IsWireframe_        = null;
  private JCheckBox IsGouraud_          = null;
  private JCheckBox IsCulling_          = null;

// ************************************************************************************
// Animation_Container constructor
// ************************************************************************************
  public Animation_Container(int num_vertices_act, IndexedGeometryArray geometry_data, Callback_PM anim)
  {
    super(num_vertices_act,geometry_data);
    this.MyAnimCB_=anim;
  }

// ************************************************************************************
// build_contentPane 
// ************************************************************************************
  public void build_contentPane()
  {
    Canvas_ = new Canvas3D_MPI_Renderer(null) { 
        Dimension prefSize = new Dimension(0, 0);
        public Dimension getPreferredSize() { return prefSize; }
    };
    Canvas_.set_animation(MyAnimCB_);


    //Create slider PERCENTAGE and its label ====================================
    JLabel sliderLabelP = new JLabel("Resolution in % VertexSplits", JLabel.CENTER);
    sliderLabelP.setAlignmentX(Component.CENTER_ALIGNMENT);

    JSlider resolution = new JSlider(JSlider.HORIZONTAL, 0, 100, PROZENT_INIT);
    resolution.addChangeListener(new SliderListener());

    resolution.setMajorTickSpacing(10); // Turn on labels at major tick marks.
    resolution.setMinorTickSpacing(5);
    resolution.setPaintTicks(true);
    resolution.setPaintLabels(true);
    resolution.setBorder(BorderFactory.createEmptyBorder(0,0,10,0));


    //Create slider FPS and its label ===========================================
    JLabel sliderLabelF = new JLabel("Required Frames/sec", JLabel.CENTER);
    sliderLabelF.setAlignmentX(Component.CENTER_ALIGNMENT);

    JSlider framesPerSecond = new JSlider(JSlider.HORIZONTAL, 0, 30, FPS_INIT);
    framesPerSecond.addChangeListener(new SliderListener());

    framesPerSecond.setMajorTickSpacing(5); // Turn on labels at major tick marks.
    framesPerSecond.setMinorTickSpacing(1);
    framesPerSecond.setPaintTicks(true);
    framesPerSecond.setPaintLabels(true);
    framesPerSecond.setBorder(BorderFactory.createEmptyBorder(0,0,10,0));


    //Create slider BPS and its label ===========================================
    JLabel sliderLabelB = new JLabel("Available Bandwidth in Mbit/sec", JLabel.CENTER);
    sliderLabelB.setAlignmentX(Component.CENTER_ALIGNMENT);

    JSlider mBitsPerSecond = new JSlider(JSlider.HORIZONTAL, 0, 20, MBPS_INIT);
    mBitsPerSecond.addChangeListener(new SliderListener());

    mBitsPerSecond.setMajorTickSpacing(2); // Turn on labels at major tick marks.
    mBitsPerSecond.setMinorTickSpacing(1);
    mBitsPerSecond.setPaintTicks(true);
    mBitsPerSecond.setPaintLabels(true);
    mBitsPerSecond.setBorder(BorderFactory.createEmptyBorder(0,0,10,0));


    // Wireframe Toggle Button =========================================================
    IsWireframe_ = new JCheckBox("Wireframe", false);
    IsWireframe_.addItemListener(new UnitsListener());
    // Backface-Culling Toggle Button ==================================================
    IsCulling_ = new JCheckBox("Culling", true);
    IsCulling_.addItemListener(new UnitsListener());
    // Shading Toggle Button ===========================================================
    IsGouraud_ = new JCheckBox("Gouraud", false);
    IsGouraud_.addItemListener(new UnitsListener());

    // left pane =========================================================== 
    JPanel leftPane = new JPanel();
    leftPane.setLayout(new BoxLayout(leftPane, BoxLayout.Y_AXIS));
    leftPane.add("Center", Canvas_);
    leftPane.add(sliderLabelP);
    leftPane.add(resolution);
    leftPane.add(sliderLabelF);
    leftPane.add(framesPerSecond);
    leftPane.add(sliderLabelB);
    leftPane.add(mBitsPerSecond);
    leftPane.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));

    //right pane =========================================================== 
    JPanel rightPane = new JPanel();
    rightPane.setLayout(new BoxLayout(rightPane, BoxLayout.Y_AXIS));
    rightPane.add(IsWireframe_);
    rightPane.add(IsGouraud_);
    rightPane.add(IsCulling_);
    rightPane.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));

    // finish the GUI =========================================================== 
    contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.X_AXIS));
    contentPane.add(leftPane);
    contentPane.add(rightPane);

    build_universe(Canvas_); // now build the content (our Mesh)
  }

// ************************************************************************************
// Inner classes
// ************************************************************************************

  /** Listens to the slider. */ 
  class SliderListener implements ChangeListener
  {
    public void stateChanged(ChangeEvent e)
    {
      JSlider source = (JSlider)e.getSource();
      // Java-Version 1.2.2 RC 3 (Blackdown) has a BUG in Swing, therefore this is commented
      //if (!source.getValueIsAdjusting())
      //{
        switch(source.getMaximum())
        {
	  case 100:
            int res = (int)source.getValue();
            System.out.println("New Resolution: " +res +"%");
            MyAnimCB_.set_resolution(res);
            break;
	  case  30:
            int fps = (int)source.getValue();
            System.out.println("New Rate: " +fps);
            MyAnimCB_.setFps(fps);
            break;
	  case  20:
            int mBps = (int)source.getValue();
            System.out.println("New BitRate: " +mBps +" MBits/sec");
            MyAnimCB_.setBps((mBps*1000000));
            break;
	  default:
            System.err.println("Legt sie um, ALLE PLATTMACHEN...");
            break;
	}
      //}
    }
  }

  /** Listens to the toggle Button. */
  class UnitsListener implements ItemListener
  {
    public void itemStateChanged(ItemEvent e)
    {
      Object source = e.getItemSelectable();
      if (source == IsWireframe_) 
      {
	if (e.getStateChange() == ItemEvent.SELECTED) 
	{
	  polygon_attributes.setPolygonMode(PolygonAttributes.POLYGON_LINE); // attached
	}
	else { polygon_attributes.setPolygonMode(PolygonAttributes.POLYGON_FILL); } // attached
      }
      else if (source == IsGouraud_) 
      {
	if (e.getStateChange() == ItemEvent.SELECTED) 
	{
	  MyAnimCB_.shading_mode(true); // Gouraud
	}
	else { MyAnimCB_.shading_mode(false); } //Flat
      }
      else if (source == IsCulling_) 
      {
	if (e.getStateChange() == ItemEvent.SELECTED) 
	{
	  polygon_attributes.setCullFace(PolygonAttributes.CULL_BACK); // Backface-Culling
	}
	else { polygon_attributes.setCullFace(PolygonAttributes.CULL_NONE); } // No Backface-Culling
      }
    }
  }
}
