package mpi_graphics;

public class Oriented_Mesh_Geometry extends Manifold_Mesh_Geometry
{
// ************************************************************************************
// Oriented_Mesh_Geometry
// ************************************************************************************
    public Oriented_Mesh_Geometry(Mesh_Info info, Mesh_Info base_info, int shade_modus)
    {
	super(info,base_info,shade_modus);
    }
        
// ************************************************************************************
// next
// ************************************************************************************
    public int next(int t_number, int v_number)
    {
	int act_pos;

	act_pos=get_vertex_position(t_number,v_number);
	act_pos=(act_pos+1)%3;

	return(triangle_neighbor_array[t_number][act_pos]);
    }

// ************************************************************************************
// previous
// ************************************************************************************
    public int previous(int t_number, int v_number)
    {
	int act_pos;

	act_pos=get_vertex_position(t_number,v_number);
	act_pos=(act_pos+2)%3;

	return(triangle_neighbor_array[t_number][act_pos]);
    }
    
// ************************************************************************************
// is_inner_vertex
// ************************************************************************************
    public boolean is_inner_vertex(int v_number)
    {
	int act_triangle_number;
	int old_triangle_number;
	int start_triangle_number;
	int start_pos;
	int act_pos_1;

	start_triangle_number=start_triangle_list[v_number];
	start_pos=start_triangle_number>>>30;
	start_triangle_number=start_triangle_number&POS_BLEND;
	act_triangle_number=start_triangle_number;

	act_pos_1=(start_pos+1)%3;

	// Suche vorwaerts
	do
	{
	    old_triangle_number=act_triangle_number;
	    act_triangle_number=triangle_neighbor_array[old_triangle_number][act_pos_1];
	    act_pos_1=pos_cache[old_triangle_number][act_pos_1];
	}
	while (act_triangle_number!=start_triangle_number && act_triangle_number!=NO_TRIANGLE);

	if (act_triangle_number==NO_TRIANGLE)
	    return false;
	    
	return true;
    }

// ************************************************************************************
// valenz_new
// ************************************************************************************
    public int valenz_new(int vertex)
    {
	int act_triangle_number;
	int old_triangle_number;
	int start_triangle_number;
	int valenz=0;
	int test_last,test_act;
	int start_pos;
	int act_pos_1;
	int act_pos_2;

	// Macht die Vertexnummer Sinn?
	if (vertex>=num_vertices_act)
		Global.Error_Occured("Vertexnummer kann nicht zur Ermittlung der Valenz benutzt werden!!!");

	start_triangle_number=start_triangle_list[vertex];
	start_pos=start_triangle_number>>>30;
	start_triangle_number=start_triangle_number&POS_BLEND;
	act_triangle_number=start_triangle_number;

	if (act_triangle_number==NO_TRIANGLE)
	    Global.Error_Occured("Dem Vertex ist kein Startdreieck zugeordnet!!!");

	act_pos_1=(start_pos+1)%3;
	act_pos_2=(start_pos+2)%3;
	test_last=get_vertex_index(act_triangle_number,act_pos_2);

	do
	{
	    valenz++;
	    old_triangle_number=act_triangle_number;
	    act_triangle_number=triangle_neighbor_array[old_triangle_number][act_pos_1];
	    act_pos_1=pos_cache[old_triangle_number][act_pos_1];

	    if (act_triangle_number==NO_TRIANGLE)
		Global.Error_Occured("Valenz eines Rand-Vertex wird in dieser Klasse nicht unterstuetzt!!!");

	    test_act=get_vertex_index(act_triangle_number,act_pos_1);
		
	    if (test_last!=test_act)
		Global.Error_Occured("Vertex-Fehler beim Valenz-Check!!!");
	    //else
	    //	System.out.println("Vertex-Check OK");

	    test_last=get_vertex_index(act_triangle_number,(act_pos_1+1)%3);
	}
	while (act_triangle_number!=start_triangle_number);

	return(valenz);
    }
        
// ************************************************************************************
// valenz
// ************************************************************************************
    public int valenz(int vertex)
    {
	int act_triangle_number;
	int start_triangle_number;
	int valenz=0;
	int test_last,test_act;
	int start_pos;
	int act_pos;

	// Macht die Vertexnummer Sinn?
	if (vertex>=num_vertices_act)
		Global.Error_Occured("Vertexnummer kann nicht zur Ermittlung der Valenz benutzt werden!!!");

	start_triangle_number=start_triangle_list[vertex]&POS_BLEND;
	act_triangle_number=start_triangle_number;

	if (act_triangle_number==NO_TRIANGLE)
	    Global.Error_Occured("Dem Vertex ist kein Startdreieck zugeordnet!!!");

	start_pos=get_vertex_position(act_triangle_number,vertex);
	act_pos=(start_pos+2)%3;
	test_last=get_vertex_index(act_triangle_number,act_pos);

	do
	{
	    valenz++;
	    act_triangle_number=next(act_triangle_number,vertex);

	    if (act_triangle_number==NO_TRIANGLE)
		Global.Error_Occured("Valenz eines Rand-Vertex wird in dieser Klasse nicht unterstuetzt!!!");

	    start_pos=get_vertex_position(act_triangle_number,vertex);
	    act_pos=(start_pos+1)%3;
	    test_act=get_vertex_index(act_triangle_number,act_pos);
		
	    if (test_last!=test_act)
		Global.Error_Occured("Vertex-Fehler beim Valenz-Check!!!");
	    //else
	    //	System.out.println("Vertex-Check OK");

	    act_pos=(start_pos+2)%3;
	    test_last=get_vertex_index(act_triangle_number,act_pos);
	}
	while (act_triangle_number!=start_triangle_number);

	return(valenz);
    }
        
// ************************************************************************************
// get_neigbors_from_inner_vertex
// ************************************************************************************
    public void get_neigbors_from_inner_vertex(int v_number, int[] t_array)
    {
	int act_triangle_number;
	int old_triangle_number;
	int start_triangle_number;
	int act_count=0;
	int start_pos;
	int act_pos_1;

	start_triangle_number=start_triangle_list[v_number];
	start_pos=start_triangle_number>>>30;
	start_triangle_number=start_triangle_number&POS_BLEND;
	act_triangle_number=start_triangle_number;

	act_pos_1=(start_pos+1)%3;

	// Suche vorwaerts
	do
	{
	    t_array[act_count]=act_triangle_number;
	    act_count++;

	    old_triangle_number=act_triangle_number;
	    act_triangle_number=triangle_neighbor_array[old_triangle_number][act_pos_1];
	    act_pos_1=pos_cache[old_triangle_number][act_pos_1];
	}
	while (act_triangle_number!=start_triangle_number && act_triangle_number!=NO_TRIANGLE);

	if (act_triangle_number==NO_TRIANGLE)
	    Global.Error_Occured("Inner Vertex != Border Vertex!!!");
	
	if (act_count!=t_array.length)
	    Global.Error_Occured("Array-Dimension ist zu gross gewaehlt!!!");
    }           
}
