/**
 * MPI-Graphics package: originaly designed and implemented by: 
 *
 * @author Robert Schneider of MPI-Saarbruecken
 *
 * further developed and changed by:
 * @author Stavros Pechlivanidis of Universitaet Bonn, Abt.IV 
 * @author Thomas Dreibholz of Universitaet Bonn, Abt.IV
 */
package mpi_graphics;

import java.io.*;
import java.lang.*;
import java.util.Date;
import javax.media.j3d.ColoringAttributes;
import bonn.infoIV.v3d2.buffer.SyncBuffer;
import bonn.infoIV.v3d2.rtp.DataChannel3D;
import bonn.infoIV.v3d2.buffer.Object3D;
import bonn.infoIV.v3d2.rtp.RTPSession; 
import bonn.infoIV.v3d2.rtp.PayloadType;

public class PM_Anim_Bonn extends Thread implements Callback_PM
{
  private static final int INIT_VALUE = -10; // just to initialize integer variables

  // variables for the Mesh itself :PMBufferStream is the pipe to the buffer
  private Binary_Sequentiel_Loader_PM_Compact PMBufferStream_ = new Binary_Sequentiel_Loader_PM_Compact();
  private Closed_Oriented_Mesh_Geometry       Mesh_           = null; // this is the displayed Mesh
  private PM_Info                             MaxMeshInfo_    = new PM_Info();   // max overall count of v.t.s
  private Mesh_Info                           BaseMeshInfo_   = new Mesh_Info(); // maxNo of v./t. in basemesh 

  // variables for the animation viewer, and all parameter needed
  private Handle_Frame            Frame_          = null; // This is the frame, holding the Animation-Viewer!
  private Animation_Container     Container_      = null;
  private Mesh_Geometry_Array     MeshArray_      = null; // The application-buffer, used by the viewer
  private PM_Info                 Info_           = null; // PM-info of the actual Frame!
  private int                     PMNumber_       = INIT_VALUE; // contains the Number of Frames in one Anim.
  private Status_Anim_Bonn        StatusCallback_ = new Status_Anim_Bonn();  //
  private Modify_Status_Anim_Bonn ModifyStatus_   = new Modify_Status_Anim_Bonn(); //

  // variables for the receiver, including the buffer 
  private SyncBuffer MyBuffer_    = null; // the buffer containing the incoming raw-data
  private RTPSession MySession_   = null; // the RTP session controling the application
  //private boolean    Running_     = true; // this is TRUE, if application is running, else FALSE
  private boolean    ActualFrame_ = true; // true:= Object.FRAME_ONE, false:= Object.FRAME_TWO
  private Thread     Producer_    = null; // this is a reference to the Producer - the receiver
  private boolean    CorrectMesh_ = false; // this is TRUE, if the read Mesh is displayable, else FALSE

  /* --- TD: Einstellungen speichern fr getFps(), getBps() --- */
  private int FramesPerSecond_     = Animation_Container.FPS_INIT;
  private int BytesPerSecond_      = (Animation_Container.MBPS_INIT * 1000000) / 8;
  private int BytesPerSecondLimit_ = (20 * 1000000) / 8;
  // --- TD: Ende --- */


  public PM_Anim_Bonn(RTPSession session)
  {
    super();
    MyBuffer_  = session.getBuffer();  // initialize MyBuffer_
    MySession_ = session;              // initialize MySession_
    Producer_  = session.getProducer();// get the reference to the producer
  }

    
  public void shutdown()
  {
    //stop();                // stop this Thread??? [DEPRACIATED]!!!
    MySession_.shutdown(); // shutdown the RTP session
    Container_ = null;     // shutdown the animation-viewer in a correct way
    if(Frame_ != null)
    {
       Frame_.dispose();
       Frame_ = null;
    }
    //Running_ = false;      // this is important for ...
  }

    
  public synchronized void waitForMe()
  {
    try { wait(); } 
    catch (InterruptedException e) { }
  }

/**
 * The run method builds up the first frame to be displayed.
 * There are three phase in this method:<br>
 * The Init-Phase: Here we initialize with the Maximum Values. We also do the Handshake with the server.<br>
 * The Read-Phase: We try to read a frame, until we can display it. We read the Frames rotatingly from our 
 * double buffered SyncBuffer. <br>
 * The View-Phase: As we have a displayable frame, we build the Viewer to show it. This takes a long time.
 */
  public void run()
  {
    // ---------------------------------- Initialize Phase -------------------------------------------
    int actualVSplitCount = INIT_VALUE; 

    // connect the SyncBuffer with this consumer-application 
    PMBufferStream_.connectToBuffer(MyBuffer_); // connect to buffer of two "Frame-arrays"
    PMNumber_ = PMBufferStream_.readPMHeaderCompactNew(MaxMeshInfo_, BaseMeshInfo_, SyncBuffer.OBJECT); 

    // now set the actual PM-Info with the maximum vertex, triangle and split count.
    Info_ = new PM_Info();
    Info_.set_vertex_count(BaseMeshInfo_.get_vertex_count());
    Info_.set_triangle_count(BaseMeshInfo_.get_triangle_count());
    Info_.set_vertex_split_count(MaxMeshInfo_.get_vertex_split_count());
    MeshArray_ = new Mesh_Geometry_Array(Info_); // build now the application-Buffer!!!

    // get the initial values of the sliders from the viewer and send them for Handshake to the sender
    int[] data = new int[3];
    data[0] = Animation_Container.FPS_INIT;
    data[1] = Animation_Container.PROZENT_INIT;
    data[2] = Animation_Container.MBPS_INIT*1000000;
    MySession_.sendApplicationMessage(PayloadType.START_TRANSFER, 1, data);
    

      synchronized (this) 
      { 
        try { wait(5); }
        catch(InterruptedException e) {}
      }
    // ---------------------------------- Read the first Frame  --------------------------------------
    while(!CorrectMesh_) // read frames, until you can display one!
    {
      //System.out.println(new Date() + " Consumer wartet auf Frame: " +ActualFrame_);
      if(MyBuffer_.getStatus(ActualFrame_, SyncBuffer.OBJECT)) ((DataChannel3D)Producer_).waitForMe();
      //System.out.println(new Date() + " Consumer fertig...");

      PMBufferStream_.read_PM_header(Info_, ActualFrame_, SyncBuffer.OBJECT); // read the info of the frame

      actualVSplitCount = MyBuffer_.getReceivedSplitCount(SyncBuffer.OBJECT, ActualFrame_);
      Info_.set_vertex_split_count(actualVSplitCount);
      MeshArray_.resize(Info_); // now resize the application-buffer according to the frame

      CorrectMesh_ = PMBufferStream_.read_pm_mesh(actualVSplitCount, MeshArray_, ActualFrame_, SyncBuffer.OBJECT); // is this frame displayable??? maybe...
      actualVSplitCount = PMBufferStream_.getActualSplitCount(); // get the actual split count

      ActualFrame_ = !ActualFrame_; // change the target Array to be read
    }
    CorrectMesh_ = false; // the Mesh is displayable, reinitialize for next frame
    System.out.print("VSplits beim ersten Male :" +actualVSplitCount); // Just an output [DB]


    // ---------------------------------- build the viewer now --------------------------------------
    ModifyStatus_.set_info(MaxMeshInfo_, BaseMeshInfo_); // sets (modifies) the status of the viewer
    Mesh_ = new Closed_Oriented_Mesh_Geometry(MaxMeshInfo_,BaseMeshInfo_,ModifyStatus_.get_shading_mode()); // build my mesh with maximum values!!!
    single_frame(actualVSplitCount, Info_, MeshArray_); // now build a single frame

    Frame_     = new Handle_Frame("Progressive4D Viewer"); // make a frame for the viewer
    Container_ = new Animation_Container(Mesh_.num_vertices_act,Mesh_.get_geometry_data(),this);
    Container_.build_contentPane(); // build the content pane for the frame in the container
    Frame_.init(Container_.get_content_pane()); // display the container in the frame

// //TOTORO Ohne Viewer
//     // Mache das Basismesh live
//     Container_ = new Animation_Container(Mesh_.num_vertices_act,Mesh_.get_geometry_data(),this);
//     Container_.build_contentPane();
//     callback_function();
// //END
      MyBuffer_.setStatus(ActualFrame_, SyncBuffer.OBJECT); // in case, the producer isn't waiting...
      //System.out.println(new Date() + " Consumer setzt Status von Frame: " +ActualFrame_ +" auf WRITE");
      synchronized (this) // as we have read the Frame, notify the Producer 
      {
        notify(); 
	//System.out.println(new Date() + " Consumer notifiziert fuer Frame: " +ActualFrame_);
      }
  }


    
// ************************************************************************************
// build_mesh
// ************************************************************************************
    public void build_mesh(int anz_splits, Mesh_Geometry_Array MeshArray_)
    {
	int i;
	boolean okey = true;
	Mesh_.load_mesh_array(MeshArray_);
        //System.out.println("MeshArray-sizes: V-Array: " +MeshArray_.vertex_array.length +" Index: " +MeshArray_.index_array_base.length);
	// Erzeuge Dreiecksnachbarschaften
	Mesh_.build_neighbour_triangle_list();
	Mesh_.build_triangle_connections();
	// Lese Vertex-Splits
	for (i=0;i<anz_splits;i++)
	{
	  //System.out.println("Vertex No: " +i);
          try
	  {
	    okey = Mesh_.add_vertex(i,MeshArray_);
            if(!okey) 
            {
              System.out.println(i +" davon angezeigt.");
              return;
	    }
	  }
	  catch(Exception e) 
          { 
	    System.out.println(i +" davon angezeigt.");
	    return; 
          }
	}
        System.out.println();
    }

// ************************************************************************************
// single_frame
// ************************************************************************************
    public void single_frame(int v_split_number,PM_Info Info_, Mesh_Geometry_Array MeshArray_)
    {
        build_mesh(v_split_number,MeshArray_);
	//System.out.println("X_MeshBuilded");
	// Berechne Normalen fuer Shading
	Mesh_.calculate_normals();
	    
	// Lade Daten in Geometrie
	Mesh_.load_vertex_indices();
    }

// ************************************************************************************
// callback_function
// ************************************************************************************
  public void callback_function() //STAVROS
  {
    // ---------------------------------- Initialize Phase -------------------------------------------
    //Status_Anim_Bonn status = StatusCallback_;
    int actualVSplitCount= INIT_VALUE; //this should be not bigger, than StatusCallback_ (Viewer-info)

    ModifyStatus_.in_callback_function(StatusCallback_);

    if (StatusCallback_.MeshResized_) // Resize my application-side Buffers and Viewer
    {
      Info_.set_vertex_count(StatusCallback_.NumVerticesMax_);
      Info_.set_triangle_count(StatusCallback_.NumTrianglesMax_);
      Info_.set_vertex_split_count(StatusCallback_.NumSplitMax_);
      Mesh_ = null; // Do we have a MEMORY-LEAK here???
      Mesh_ = new Closed_Oriented_Mesh_Geometry(Info_, BaseMeshInfo_, StatusCallback_.Shading_); // build mesh
      Container_.set_new_geometry(Mesh_.get_geometry_data()); // set the geometry, but without NEW (I think)
    }


    // ---------------------------------- Read the next Frame  --------------------------------------
    while(!CorrectMesh_)
    {
      //System.out.println(new Date() + " Consumer wartet auf Frame: " + ActualFrame_);
      if(MyBuffer_.getStatus(ActualFrame_, SyncBuffer.OBJECT)) ((DataChannel3D)Producer_).waitForMe();
      //System.out.println(new Date() + " Consumer fertig...");

      PMBufferStream_.read_PM_header(Info_, ActualFrame_, SyncBuffer.OBJECT);

      actualVSplitCount = MyBuffer_.getReceivedSplitCount(SyncBuffer.OBJECT, ActualFrame_);
      Info_.set_vertex_split_count(actualVSplitCount);
      MeshArray_.resize(Info_);

      CorrectMesh_ = PMBufferStream_.read_pm_mesh(actualVSplitCount, MeshArray_, ActualFrame_, SyncBuffer.OBJECT); // is this frame displayable?
      actualVSplitCount = PMBufferStream_.getActualSplitCount(); // this is the actual read VSplit count

      MyBuffer_.setStatus(ActualFrame_, SyncBuffer.OBJECT);
      //System.out.println(new Date() + " Consumer setzt Status von Frame: " + ActualFrame_ +" auf WRITE");
      synchronized (this) // as we have read the Frame, notify the Producer 
      {
        notify(); 
	//System.out.println(new Date() + " Consumer notifiziert fuer Frame: " +ActualFrame_);
      }
      ActualFrame_ = !ActualFrame_; // change the target Array to be read
    }
    CorrectMesh_ = false; // the Mesh is displayable, reinitialize for next frame
    System.out.print("VSplits :" +actualVSplitCount +"... "); // Just an output [DB]


    // ---------------------------------- display the frame --------------------------------------
    single_frame(actualVSplitCount, Info_, MeshArray_);

// //TOTORO Ohne Viewer!
//     callback_function();
// //END
  }

// ************************************************************************************
// set_resolution
// ************************************************************************************
    public void set_resolution(float prozent)
    {
	ModifyStatus_.in_set_resolution(prozent);
	
	// So hier kann gleichzeitig die Information an den Client
	// weitergeleitet werden.
    }

  public void set_resolution(int prozent)//STAVROS
  {
    MySession_.sendApplicationMessage(PayloadType.SPLITS_PERCENTAGE, prozent, null);
  }

  public void setFps(int fps)//STAVROS
  {
    MySession_.sendApplicationMessage(PayloadType.FRAMES_PER_SEC, fps, null);
  }

  public void setBps(int bps)//SP & TD
  {
    BytesPerSecond_ = bps / 8;

    if(BytesPerSecond_ > BytesPerSecondLimit_) {
       bps = BytesPerSecondLimit_ * 8;
    }

    // System.out.println("BPS: Wert="+bps/8+"  Limit="+BytesPerSecondLimit_+"  Wanted="+BytesPerSecond_); 

    MySession_.sendApplicationMessage(PayloadType.BITS_PER_SEC, bps, null);
  }


  public int getFps() {
     return(FramesPerSecond_);
  }

  public int getBps() {
     return(BytesPerSecond_ * 8);
  }

  public int getBpsLimit() {
     return(BytesPerSecondLimit_ * 8);
  }

  public void setBpsLimit(int limit) {
     BytesPerSecondLimit_ = limit / 8;
     setBps(BytesPerSecond_ * 8);
  }

  public void startTransfer(int loop, int[] data)//STAVROS
  {
    MySession_.sendApplicationMessage(PayloadType.START_TRANSFER, loop, data);
  }


// ************************************************************************************
// shading_mode
// ************************************************************************************
    public void shading_mode(boolean is_gouraud)
    {
	if (is_gouraud==true)
	    ModifyStatus_.in_shading_mode(ColoringAttributes.SHADE_GOURAUD);
	else
	    ModifyStatus_.in_shading_mode(ColoringAttributes.SHADE_FLAT);
	    
	Mesh_.redraw();
    }
}



