#include <RTP/RTCP_Packet.hh>

RTCP_Common_Header::RTCP_Common_Header()
{
  setVersion(RTP_VERSION);
  setPadding(false);
  setCount(0);
}

RTCP_Report::RTCP_Report()
{
  setPacketType(0);
  setLength((sizeof(*this) / 4) - 1);
  setSSRC(0);
}

RTCP_Sender_Report::RTCP_Sender_Report()
{
  setPacketType(RTCP_SR);
  setLength((sizeof(*this) / 4) - 1);
}

RTCP_Receiver_Report::RTCP_Receiver_Report()
{
  setPacketType(RTCP_RR);
  setLength((sizeof(*this) / 4) - 1);
}

RTCP_Source_Description::RTCP_Source_Description()
{
  setPacketType(RTCP_SDES);
  setLength((sizeof(*this) / 4) - 1);
}

RTCP_Bye::RTCP_Bye()
{
  setPacketType(RTCP_BYE);
  setLength((sizeof(*this) / 4) - 1);
}

RTCP_App::RTCP_App()
{
  setPacketType(RTCP_APP);
  setLength((sizeof(*this) / 4) - 1);
}

RTCP_Union_Packet::RTCP_Union_Packet()
{
  setPacketType(0);
  setLength((sizeof(*this) / 4) - 1);
}

u_int32 RTCP_Common_Header::getVersion()    { return v;            }
u_int32 RTCP_Common_Header::getPadding()    { return p;            }
u_int32 RTCP_Common_Header::getCount()      { return c;            }
u_int32 RTCP_Common_Header::getPacketType() { return pt;           }
u_int16 RTCP_Common_Header::getLength()     { return reverse(len); }

void RTCP_Common_Header::setVersion(u_int32 version)       { v   = version;         }
void RTCP_Common_Header::setPadding(u_int32 padding)       { p   = padding;         }
void RTCP_Common_Header::setCount(u_int32 count)           { c   = count;           }
void RTCP_Common_Header::setPacketType(u_int32 packetType) { pt  = packetType;      }
void RTCP_Common_Header::setLength(u_int16 length)         { len = reverse(length); }

RTCP_Sender_Info_Block::RTCP_Sender_Info_Block() { }

u_int32 RTCP_Sender_Info_Block::getNTPSeconds()   { return ntp_sec;  }
u_int32 RTCP_Sender_Info_Block::getNTPFraction()  { return ntp_frac; }
u_int32 RTCP_Sender_Info_Block::getRTPTimeStamp() { return rtp_ts;   }
u_int32 RTCP_Sender_Info_Block::getPacketsSent()  { return p_sent;   }
u_int32 RTCP_Sender_Info_Block::getOctetsSent()   { return o_sent;   }

void RTCP_Sender_Info_Block::setNTPSeconds(u_int32 seconds)   { ntp_sec  = seconds;  }
void RTCP_Sender_Info_Block::setNTPFraction(u_int32 fraction) { ntp_frac = fraction; }
void RTCP_Sender_Info_Block::setRTPTimeStamp(u_int32 ts)      { rtp_ts   = ts;       }
void RTCP_Sender_Info_Block::setPacketsSent(u_int32 packets)  { p_sent   = packets;  }
void RTCP_Sender_Info_Block::setOctetsSent(u_int32 octets)    { o_sent   = octets;   }

RTCP_Reception_Report_Block::RTCP_Reception_Report_Block()             { }
RTCP_Reception_Report_Block::RTCP_Reception_Report_Block(u_int32 ssrc) { }

u_int32 RTCP_Reception_Report_Block::getSSRC()         { return reverse(ssrc);     }
float   RTCP_Reception_Report_Block::getFractionLost() { return fraction / 256.0;  }
u_int32 RTCP_Reception_Report_Block::getPacketsLost()  { return lost;              }
u_int32 RTCP_Reception_Report_Block::getLastSeqNum()   { return reverse(last_seq); }
u_int32 RTCP_Reception_Report_Block::getJitter()       { return reverse(jitter);   }
u_int32 RTCP_Reception_Report_Block::getLSR()          { return reverse(lsr);      }
u_int32 RTCP_Reception_Report_Block::getDLSR()         { return reverse(dlsr);     }

void RTCP_Reception_Report_Block::setSSRC(u_int32 syncSource)    { ssrc     = reverse(syncSource); }
void RTCP_Reception_Report_Block::setFractionLost(u_int32 frac)  { fraction = frac;                }
void RTCP_Reception_Report_Block::setPacketsLost(u_int32 Lost)   { lost     = Lost;                }
void RTCP_Reception_Report_Block::setLastSeqNum(u_int32 LastSeq) { last_seq = reverse(LastSeq);    }
void RTCP_Reception_Report_Block::setJitter(u_int32 Jitter)      { jitter   = reverse(Jitter);     }
void RTCP_Reception_Report_Block::setLSR(u_int32 LSR)            { lsr      = reverse(LSR);        }
void RTCP_Reception_Report_Block::setDLSR(u_int32 DLSR)          { dlsr     = reverse(DLSR);       }

u_int32 RTCP_Report::getSSRC() { return reverse(ssrc); }

void RTCP_Report::setSSRC(u_int32 syncSource) { ssrc = reverse(syncSource); }
