// RTP_Session.hh --- RTP Session Class Header File

#ifndef RTP_SESSION_H
#define RTP_SESSION_H

#include <iostream.h>          // cout, cerr
#include <pthread.h>           // pthread
#include <stl_config.h>        // bool
#include <time/time.hh>        // Time
#include <socket/socket.hh>    // RTPSocket/RTCPSocket
#include <RTP/SSI.hh>
#include <RTP/RTP_Packet.hh>
#include <RTP/RTCP_Packet.hh>

extern Socket RTPSocket, RTCPSocket, FECSocket;

class RTP_Session
{
private:
  pthread_t rtp_send_thread_id;
  pthread_t rtp_recv_thread_id;
  pthread_attr_t rtp_send_thread_attr;
  pthread_attr_t rtp_recv_thread_attr;
  pthread_t rtcp_send_thread_id;
  pthread_t rtcp_recv_thread_id;
  pthread_attr_t rtcp_send_thread_attr;
  pthread_attr_t rtcp_recv_thread_attr;
public:
  RTP_Session();

// TD: Compiler-Warnung wg. virt. Funktionen und nicht-virtuellem Destruktor.
  virtual ~RTP_Session();
// TD: Ende.

  pthread_t transmit();
  pthread_t join();
  void finish();
  void leave();
  void susp();
  void cont();
private:
  friend void* rtp_send_thread(void*);
  friend void* rtp_recv_thread(void*);
  friend void* rtcp_send_thread(void*);
  friend void* rtcp_recv_thread(void*);
private:
  SSI_Table SSITable;
private:
  bool   shutdown;                        // True, if rtcp protocol has to shutdown
  bool   initial;                         // True, if application has not yet sent an RTCP packet
  bool   we_sent;                         // True, if data sent since 2nd previous RTCP report
  int    senders;                         // Most current estimate for nr. of session senders
  int    members;                         // Most current estimate for nr. of session members
  int    pmembers;                        // Estimated nr. of session members at time tp
  double rtcp_bw;                         // Bandwidth for RTCP in octets/second
  double avg_rtcp_size;                   // Average compound RTCP packet size
  Time   tp;                              // The last time an RTCP packet was transmitted
  Time   tc;                              // The current time
  Time   tn;                              // The next scheduled transmission time of an RTCP packet
  Time   ts;                              // The time to sleep until next RTCP packet
protected:
  void stop(RTCP_Bye*);
private:
  void sendLoopRTCP();
  void recvLoopRTCP();
protected:
  unsigned short send(RTCP_Packet*, int trafficClass);
  unsigned short recv(RTCP_Packet*);
private:
  double transInterval();
  virtual void feedback(RTCP_Packet*) = 0;
private:
  unsigned int ssrc;                      // The SSRC Identifier
  unsigned short sequence_nr;             // The current sequence number
private:
  virtual void sendLoopRTP() = 0;
  virtual void recvLoopRTP() = 0;
protected:
  unsigned short send(RTP_Packet*, unsigned short, int trafficClass);
  unsigned short recv(RTP_Packet*, unsigned short);
private:
  unsigned short encode(RTP_Packet*, unsigned short);
  unsigned short decode(RTP_Packet*, unsigned short);
  unsigned short reconstruct(RTP_Packet*, RTP_Packet*, unsigned short, FEC_Packet*, unsigned short);
  void send(FEC_Packet*, unsigned short, unsigned short, unsigned int, unsigned short, int trafficClass);
  void reset(RTP_Packet*, unsigned short);
  void reset(FEC_Packet*, unsigned short);
  void transit(RTP_Packet*, RTP_Packet*, unsigned short);
  void XOR(FEC_Packet*, RTP_Packet*, unsigned short);
  void XOR(RTP_Packet*, RTP_Packet*, unsigned short);
  void XOR(RTP_Packet*, FEC_Packet*, unsigned short);
  enum FEC_State { NO_RECOVER, RECOVER_BASE, RECOVER_EXT, RECOVERED };
};

#endif
