#include <gui.hh>

MenuBar::MenuBar(QWidget *parent, const char *name)
       : QMenuBar(parent, name)
{
  /*
   * Automatically sets its own geometry to the top of
   * the parent widget and changes appropriately it when
   * the parent widget is resized.
   */

  QPopupMenu *fileMenu = new QPopupMenu();

  fileMenu->insertItem("&Open");
  fileMenu->insertSeparator();
  fileMenu->insertItem("&Quit");

  QPopupMenu *optionsMenu = new QPopupMenu();

  optionsMenu->insertItem("&Advanced");

  QPopupMenu *helpMenu = new QPopupMenu();

  helpMenu->insertItem("&Index");
  helpMenu->insertSeparator();
  helpMenu->insertItem("&About");

  insertItem("&File", fileMenu);
  insertItem("&Options", optionsMenu);
  insertItem("&Help", helpMenu);
}

ToolBar::ToolBar(QMainWindow *parent, const char *name)
       : QToolBar(parent, name)
{
  QToolButton *openToolButton = new QToolButton(this);

  openToolButton->setPixmap(openPixmap);

  QToolButton *closeToolButton = new QToolButton(this);

  closeToolButton->setPixmap(closePixmap);

  QToolButton *helpToolButton = new QToolButton(this);

  helpToolButton->setPixmap(helpPixmap);
}

StatusBar::StatusBar(QWidget *parent, const char *name)
         : QStatusBar(parent, name)
{
//setGeometry(0, 378, 400, 100);

  // Display message in status bar.
  message("This is the status bar.");
}

TransportAddressWidget::TransportAddressWidget(QString title, QString hostname, QString ipaddress,
					       QString port, QWidget *parent, const char *name)
                      : QGroupBox(title, parent, name)
{
  this->setAlignment(AlignHCenter);

  hostname_label = new QLabel("Hostname", this);
  ipaddress_label = new QLabel("IP-Address", this);
  port_label = new QLabel("Port", this);

  hostname_val = new QLabel(hostname, this);
  ipaddress_val = new QLabel(ipaddress, this);
  port_val = new QLabel(port, this);

  glayout = new QGridLayout(this, 3, 2, 10, 0);

  glayout->addWidget(hostname_label, 0, 0);
  glayout->addWidget(ipaddress_label, 1, 0);
  glayout->addWidget(port_label, 2, 0);
  glayout->addWidget(hostname_val, 0, 1);
  glayout->addWidget(ipaddress_val, 1, 1);
  glayout->addWidget(port_val, 2, 1);
}

TransportAddressesWidget::TransportAddressesWidget(QWidget *parent, const char *name)
                        : QWidget(parent, name)
{
  local_host = new TransportAddressWidget("Local Host", "gaffel.cs.uni-bonn.de",
					  "110.53.223.1", "5000/5001", this);

  remote_host = new TransportAddressWidget("Remote Host", "diebles.cs.uni-bonn.de",
					   "110.53.223.98", "6000/6001", this);

  glayout = new QGridLayout(this, 1, 2, 10, 10);

  glayout->addWidget(local_host, 0, 0);
  glayout->addWidget(remote_host, 0, 1);
}

FileStatusWidget::FileStatusWidget(QWidget *parent, const char *name)
                : QWidget(parent, name)
{
  f_snt = new QLCDNumber(4, this);
  f_rem = new QLCDNumber(4, this);
  f_tot = new QLCDNumber(4, this);
  f_all = new QLCDNumber(5, this);

  glayout = new QGridLayout(this, 1, 4, 10, 10);

  glayout->addWidget(f_snt, 0, 0);
  glayout->addWidget(f_rem, 0, 1);
  glayout->addWidget(f_tot, 0, 2);
  glayout->addWidget(f_all, 0, 3);
}

TabDialog::TabDialog(QWidget *parent, const char *name)
         : QTabDialog(parent, name)
{
  transportAddr = new TransportAddressesWidget(this);
  fileStatus = new FileStatusWidget(this);

  addTab(transportAddr, "Transport Addresses");
  addTab(fileStatus, "File Status");

  // Create the menu bar.
//menuBar = new MenuBar(this);

  // Create the tool bar.
//toolBar = new ToolBar(this);

  // Create the status bar.
//statusBar = new StatusBar(this);
}
