#include <iostream.h>       // cout, cerr, ...
#include <netdb.h>          // gethostbyname()
#include <netinet/in.h>     // struct sockaddr
#include <session.hh>
#include <socket/socket.hh>

Socket RTPSocket, RTCPSocket, FECSocket;

int main(int argc, char *argv[])
{
  unsigned short local_port;
  unsigned short remote_port;

  if (argc != 4)
    {
      cerr << "\nusage: " << argv[0] << " <local_port> <remote_hostname> <remote_port>\n"
	   << "\t<local_hostname> and <remote_hostname> is either\n"
	   << "\t\t- a hostname, or\n"
	   << "\t\t- an IPv4 address in standard dot notation, or\n"
	   << "\t\t- an IPv6 address in colon (and possibly dot) notation\n"
	   << "\t<local_port> and <remote_port> is a port number with a\n"
	   << "\t\t- minimum value of " << RTP_MIN_PORT << "\n"
	   << "\t\t- maximum value of " << RTP_MAX_PORT << "\n"
	   << endl;
      exit(EXIT_FAILURE);
    }
  else
    {
      local_port = atoi(argv[1]);
      remote_port = atoi(argv[3]);
    }

  sockaddr_in local_addr;
  local_addr.sin_family = AF_INET;
  if ((local_port < RTP_MIN_PORT) || (local_port > RTP_MAX_PORT))
    {
      cerr << "main(): Local port out of range\n";
      exit(EXIT_FAILURE);
    }
  else local_addr.sin_port = htons(local_port);
  local_addr.sin_addr.s_addr = htonl(INADDR_ANY);

  sockaddr_in remote_addr;
  remote_addr.sin_family = AF_INET;
  if ((remote_port < RTP_MIN_PORT) || (remote_port > RTP_MAX_PORT))
    {
      cerr << "main(): Remote port out of range\n";
      exit(EXIT_FAILURE);
    }
  else remote_addr.sin_port = htons(remote_port);
  struct hostent *hp = gethostbyname(argv[2]);
  // hp == NULL on failure
  if (hp == NULL)
    {
      perror("gethostbyname()");
      exit(EXIT_FAILURE);
    }
  // hp != NULL on success
  else
    {      
      remote_addr.sin_family = hp->h_addrtype;
      memcpy(&remote_addr.sin_addr.s_addr, hp->h_addr, hp->h_length);
    }

  /***********************************************************************************************/

  RTPSocket.create();
  RTPSocket.assignName((struct sockaddr *) &local_addr, sizeof(local_addr));
  RTPSocket.initConnection((struct sockaddr *) &remote_addr, sizeof(remote_addr));

  local_addr.sin_port  = htons(local_port  + 1);
  remote_addr.sin_port = htons(remote_port + 1);

  RTCPSocket.create();
  RTCPSocket.assignName((struct sockaddr *) &local_addr, sizeof(local_addr));
  RTCPSocket.initConnection((struct sockaddr *) &remote_addr, sizeof(remote_addr));

  local_addr.sin_port  = htons(local_port  + 2);
  remote_addr.sin_port = htons(remote_port + 2);

  FECSocket.create();
  FECSocket.assignName((struct sockaddr *) &local_addr, sizeof(local_addr));
  FECSocket.initConnection((struct sockaddr *) &remote_addr, sizeof(remote_addr));


  // TD: NULL-Zeiger fr QoS-Manager (nur fr Server sinnvoll)
  Prog4D_Session session(NULL);


  pthread_join(session.join(), NULL);

  return 0;
}
