// ##########################################################################
// ####                                                                  ####
// ####                      RTP Audio Server Project                    ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### Strings Implementation                                           ####
// ####                                                                  ####
// #### Version 1.00  --  February 23, 2001                              ####
// ####                                                                  ####
// #### Copyright (C) 1999  Thomas Dreibholz                             ####
// ####               2000  Universitt Bonn, Abt. IV                    ####
// ####               2001  EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################


#include "system.h"
#include "strings.h"


#include <ctype.h>


// ###### Constructor #######################################################
String::String()
{
   setData(NULL);
}


// ###### Constructor #######################################################
String::String(const String& string)
{
   setData(stringDuplicate(string.getData()));
}


// ###### Constructor #######################################################
String::String(const char* string)
{
   setData(stringDuplicate(string));
}


// ###### Constructor #######################################################
String::String(const char* string, const cardinal length)
{
   if(string != NULL) {
      char str[length + 1];
      memcpy((void*)&str,string,length);
      str[length] = 0x00;
      setData(stringDuplicate((char*)&str));
   }   
   else
      setData(NULL);
}


// ###### Constructor #######################################################
String::String(const cardinal value)
{
   char string[64];
   snprintf((char*)&string,sizeof(string),"%Ld",(card64)value);
   setData(stringDuplicate((char*)&string));
}


// ###### Destructor ########################################################
String::~String()
{
   delete [] getData();
}


// ###### "="-operator ######################################################
String& String::operator=(const String& string)
{
   if(this != &string) {
      delete [] getData();
      setData(stringDuplicate(string.getData()));
   }
   return(*this);
}


// ###### "="-operator ######################################################
String& String::operator=(const char* string)
{
   delete [] getData();
   setData(stringDuplicate(string));
   return(*this);
}


// ###### "="-operator ######################################################
String& String::operator=(const cardinal value)
{
   delete [] getData();
   char string[64];
   snprintf((char*)&string,sizeof(string),"%d",value);
   setData(stringDuplicate((char*)&string));
   return(*this);
}


// ###### Convert string to lowercase #######################################
String String::toLower() const
{
   const char* src    = getData();
   const cardinal len = length();
   char buffer[len + 1];
   
   cardinal i;
   for(i = 0;i < len;i++) {
      buffer[i] = tolower(src[i]);
   }
   buffer[i] = 0x00;
   return((char*)&buffer);
}


// ###### Convert string to uppercase #######################################
String String::toUpper() const
{
   const char* src    = getData();
   const cardinal len = length();
   char buffer[len + 1];
   
   cardinal i;
   for(i = 0;i < len;i++) {
      buffer[i] = toupper(src[i]);
   }
   buffer[i] = 0x00;
   return((char*)&buffer);
}


// ###### Get left part of string ###########################################
String String::left(const cardinal maxChars) const
{
   const char* src    = getData();
   const cardinal len = min(length(),maxChars);
   char buffer[len + 1];
   
   cardinal i;
   for(i = 0;i < len;i++) {
      buffer[i] = src[i];
   }
   buffer[i] = 0x00;
   return((char*)&buffer);
}


// ###### Get middle part of string #########################################
String String::mid(const cardinal start, const cardinal maxChars) const
{
   const cardinal strlen = length();
   if(strlen <= start)
      return((char*)"");
   
   const char* src  = getData();
   const cardinal len = min(strlen - start,maxChars);
   char buffer[len + 1];
   
   cardinal i;
   for(i = 0;i < len;i++) {
      buffer[i] = src[i + start];
   }
   buffer[i] = 0x00;
   return((char*)&buffer);
}


// ###### Get right part of string ##########################################
String String::right(const cardinal maxChars) const
{
   const cardinal strlen = length();
   const char*    src    = getData();
   const cardinal len    = min(strlen,maxChars);
   char buffer[len + 1];
   
   cardinal i,j;
   for(i = 0, j = strlen - len;i < len;i++,j++) {
      buffer[i] = src[j];
   }
   buffer[i] = 0x00;
   return((char*)&buffer);
}


// ###### Remove spaces #####################################################
String String::stripWhiteSpace() const
{
   integer length = strlen(Data);
   integer i = 0;
   while((Data[i] == ' ') && (i < length)) {
      i++;
   }
   integer j = length - 1;
   while((Data[j] == ' ') && (j >= i)) {
      j--;
   }
   return(mid(i,j - i + 1));
}


// ###### Scan setting string ###############################################
bool String::scanSetting(String& s1, String& s2) const
{
   integer found = index('=');
   if(found >= 1) {
      s1 = left(found).stripWhiteSpace().toUpper();
      s2 = right(length() - found - 1).stripWhiteSpace();
      if((s1.length() > 0) && (s2.length() > 0)) {
         const char* data      = s2.getData();
         const cardinal length = s2.length();
         if((data[0] == '\"') && (data[length - 1] == '\"')) {
            s2 = s2.mid(1,length - 2);
         }
         return(true);
      }
   }
   return(false);
}


// ###### "<<"-operator #####################################################
ostream& operator<<(ostream& os, const String& string)
{
   os << string.getData();
   return(os);
}


// ###### "+"-operator ######################################################
String operator+(const String& string1, const String& string2)
{
   char str[string1.length() + string2.length() + 1];
   strcpy((char*)&str,string1.getData());
   strcat((char*)&str,string2.getData());
   return String((const char*)&str);
}
